@@ -1,14 +1,15 @@
 package com.fasterxml.jackson.core.base;
 
-import java.io.*;
-import java.math.BigDecimal;
-
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.json.DupDetector;
 import com.fasterxml.jackson.core.json.JsonWriteContext;
 import com.fasterxml.jackson.core.json.PackageVersion;
 import com.fasterxml.jackson.core.util.DefaultPrettyPrinter;
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.math.BigDecimal;
+
 /**
  * This base class implements part of API that a JSON generator exposes
  * to applications, adds shared internal methods that sub-classes
@@ -27,6 +28,7 @@ public abstract class GeneratorBase extends JsonGenerator
      * 
      * @since 2.5
      */
+    @SuppressWarnings("deprecation")
     protected final static int DERIVED_FEATURES_MASK =
             Feature.WRITE_NUMBERS_AS_STRINGS.getMask()
             | Feature.ESCAPE_NON_ASCII.getMask()
@@ -51,7 +53,7 @@ public abstract class GeneratorBase extends JsonGenerator
      * @since 2.7.7
      */
     protected final static int MAX_BIG_DECIMAL_SCALE = 9999;
-    
+
     /*
     /**********************************************************
     /* Configuration
@@ -70,7 +72,7 @@ public abstract class GeneratorBase extends JsonGenerator
     /**
      * Flag set to indicate that implicit conversion from number
      * to JSON String is needed (as per
-     * {@link com.fasterxml.jackson.core.JsonGenerator.Feature#WRITE_NUMBERS_AS_STRINGS}).
+     * {@link com.fasterxml.jackson.core.json.JsonWriteFeature#WRITE_NUMBERS_AS_STRINGS}).
      */
     protected boolean _cfgNumbersAsStrings;
 
@@ -99,6 +101,7 @@ public abstract class GeneratorBase extends JsonGenerator
     /**********************************************************
      */
 
+    @SuppressWarnings("deprecation")
     protected GeneratorBase(int features, ObjectCodec codec) {
         super();
         _features = features;
@@ -109,9 +112,8 @@ public abstract class GeneratorBase extends JsonGenerator
         _cfgNumbersAsStrings = Feature.WRITE_NUMBERS_AS_STRINGS.enabledIn(features);
     }
 
-    /**
-     * @since 2.5
-     */
+    // @since 2.5
+    @SuppressWarnings("deprecation")
     protected GeneratorBase(int features, ObjectCodec codec, JsonWriteContext ctxt) {
         super();
         _features = features;
@@ -124,6 +126,9 @@ public abstract class GeneratorBase extends JsonGenerator
      * Implemented with standard version number detection algorithm, typically using
      * a simple generated class, with information extracted from Maven project file
      * during build.
+     *
+     * @return Version number of the generator (version of the jar that contains
+     *     generator implementation class)
      */
     @Override public Version version() { return PackageVersion.VERSION; }
 
@@ -134,7 +139,9 @@ public abstract class GeneratorBase extends JsonGenerator
 
     @Override
     public void setCurrentValue(Object v) {
-        _writeContext.setCurrentValue(v);
+        if (_writeContext != null) {
+            _writeContext.setCurrentValue(v);
+        }
     }
 
     /*
@@ -149,6 +156,7 @@ public abstract class GeneratorBase extends JsonGenerator
 
     //public JsonGenerator configure(Feature f, boolean state) { }
 
+    @SuppressWarnings("deprecation")
     @Override
     public JsonGenerator enable(Feature f) {
         final int mask = f.getMask();
@@ -168,6 +176,7 @@ public abstract class GeneratorBase extends JsonGenerator
         return this;
     }
 
+    @SuppressWarnings("deprecation")
     @Override
     public JsonGenerator disable(Feature f) {
         final int mask = f.getMask();
@@ -216,6 +225,7 @@ public abstract class GeneratorBase extends JsonGenerator
      *
      * @since 2.7
      */
+    @SuppressWarnings("deprecation")
     protected void _checkStdFeatureChanges(int newFeatureFlags, int changedFeatures)
     {
         if ((changedFeatures & DERIVED_FEATURES_MASK) == 0) {
@@ -283,10 +293,9 @@ public abstract class GeneratorBase extends JsonGenerator
     public void writeStartObject(Object forValue) throws IOException
     {
         writeStartObject();
-        if ((_writeContext != null) && (forValue != null)) {
-            _writeContext.setCurrentValue(forValue);
+        if (forValue != null) {
+            setCurrentValue(forValue);
         }
-        setCurrentValue(forValue);
     }
 
     /*
@@ -298,11 +307,13 @@ public abstract class GeneratorBase extends JsonGenerator
     @Override public void writeFieldName(SerializableString name) throws IOException {
         writeFieldName(name.getValue());
     }
-    
+
     //public abstract void writeString(String text) throws IOException;
 
     //public abstract void writeString(char[] text, int offset, int len) throws IOException;
 
+    //public abstract void writeString(Reader reader, int len) throws IOException;
+
     //public abstract void writeRaw(String text) throws IOException,;
 
     //public abstract void writeRaw(char[] text, int offset, int len) throws IOException;
@@ -340,10 +351,10 @@ public abstract class GeneratorBase extends JsonGenerator
     }
 
     /*
-   /**********************************************************
-   /* Public API, write methods, primitive
-   /**********************************************************
-    */
+    /**********************************************************
+    /* Public API, write methods, primitive
+    /**********************************************************
+     */
 
     // Not implemented at this level, added as placeholders
 
@@ -424,6 +435,9 @@ public abstract class GeneratorBase extends JsonGenerator
      * 
      * @param typeMsg Additional message used for generating exception message
      *   if value output is NOT legal in current generator output state.
+     *
+     * @throws IOException if there is either an underlying I/O problem or encoding
+     *    issue at format layer
      */
     protected abstract void _verifyValueWrite(String typeMsg) throws IOException;
 
@@ -431,6 +445,8 @@ public abstract class GeneratorBase extends JsonGenerator
      * Overridable factory method called to instantiate an appropriate {@link PrettyPrinter}
      * for case of "just use the default one", when {@link #useDefaultPrettyPrinter()} is called.
      *
+     * @return Instance of "default" pretty printer to use
+     *
      * @since 2.6
      */
     protected PrettyPrinter _constructDefaultPrettyPrinter() {
@@ -441,6 +457,12 @@ public abstract class GeneratorBase extends JsonGenerator
      * Helper method used to serialize a {@link java.math.BigDecimal} as a String,
      * for serialization, taking into account configuration settings
      *
+     * @param value BigDecimal value to convert to String
+     *
+     * @return String representation of {@code value}
+     *
+     * @throws IOException if there is a problem serializing value as String
+     *
      * @since 2.7.7
      */
     protected String _asString(BigDecimal value) throws IOException {
@@ -463,14 +485,13 @@ scale, MAX_BIG_DECIMAL_SCALE, MAX_BIG_DECIMAL_SCALE));
     /**********************************************************
      */
 
-    /**
-     * @since 2.5
-     */
+    // @since 2.5
     protected final int _decodeSurrogate(int surr1, int surr2) throws IOException
     {
         // First is known to be valid, but how about the other?
         if (surr2 < SURR2_FIRST || surr2 > SURR2_LAST) {
-            String msg = "Incomplete surrogate pair: first char 0x"+Integer.toHexString(surr1)+", second 0x"+Integer.toHexString(surr2);
+            String msg = String.format(
+"Incomplete surrogate pair: first char 0x%04X, second 0x%04X", surr1, surr2);
             _reportError(msg);
         }
         int c = 0x10000 + ((surr1 - SURR1_FIRST) << 10) + (surr2 - SURR2_FIRST);
