@@ -2,10 +2,12 @@ package com.fasterxml.jackson.core.filter;
 
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.Reader;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.filter.TokenFilter.Inclusion;
 import com.fasterxml.jackson.core.util.JsonGeneratorDelegate;
 
 /**
@@ -17,6 +19,7 @@ import com.fasterxml.jackson.core.util.JsonGeneratorDelegate;
  */
 public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 {
+
     /*
     /**********************************************************
     /* Configuration
@@ -44,18 +47,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
      * done and only explicitly included entries are output; if `true` then
      * path from main level down to match is also included as necessary.
      */
-    protected boolean _includePath;
-
-    /* NOTE: this feature is included in the first version (2.6), but
-     * there is no public API to enable it, yet, since there isn't an
-     * actual use case. But it seemed possible need could arise, which
-     * is feature has not yet been removed. If no use is found within
-     * first version or two, just remove.
-     * 
-     * Marked as deprecated since its status is uncertain.
-     */
-    @Deprecated
-    protected boolean _includeImmediateParent;
+    protected TokenFilter.Inclusion _inclusion;
 
     /*
     /**********************************************************
@@ -89,8 +81,32 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     /**********************************************************
      */
 
+    /**
+     * @param d Generator to delegate calls to
+     * @param f Filter to use
+     * @param includePath Whether to include path or not
+     * @param allowMultipleMatches Whether to allow multiple matches
+     *
+     * @deprecated since 2.12 Use the constructor that takes {@link TokenFilter.Inclusion}
+     *    argument instead.
+     */
+    @Deprecated
     public FilteringGeneratorDelegate(JsonGenerator d, TokenFilter f,
             boolean includePath, boolean allowMultipleMatches)
+    {
+        this(d, f, includePath ? Inclusion.INCLUDE_ALL_AND_PATH : Inclusion.ONLY_INCLUDE_ALL, allowMultipleMatches);
+    }
+
+    /**
+     * @param d Generator to delegate calls to
+     * @param f Filter to use
+     * @param inclusion Definition of inclusion criteria
+     * @param allowMultipleMatches Whether to allow multiple matches
+     *
+     * @since 2.12
+     */
+    public FilteringGeneratorDelegate(JsonGenerator d, TokenFilter f,
+            TokenFilter.Inclusion inclusion, boolean allowMultipleMatches)
     {
         // By default, do NOT delegate copy methods
         super(d, false);
@@ -98,7 +114,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
         // and this is the currently active filter for root values
         _itemFilter = f;
         _filterContext = TokenFilterContext.createRootContext(f);
-        _includePath = includePath;
+        _inclusion = inclusion;
         _allowMultipleMatches = allowMultipleMatches;
     }
 
@@ -117,6 +133,8 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     /**
      * Accessor for finding number of matches, where specific token and sub-tree
      * starting (if structured type) are passed.
+     *
+     * @return Number of matches
      */
     public int getMatchCount() {
         return _matchCount;
@@ -169,11 +187,16 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
             delegate.writeStartArray();
+        } else if (_itemFilter != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray();
         } else {
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
         }
     }
-        
+
+    @SuppressWarnings("deprecation")
     @Override
     public void writeStartArray(int size) throws IOException
     {
@@ -198,6 +221,68 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
             delegate.writeStartArray(size);
+        } else if (_itemFilter != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(size);
+        } else {
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
+        }
+    }
+
+    @Override
+    public void writeStartArray(Object forValue) throws IOException
+    {
+        if (_itemFilter == null) {
+            _filterContext = _filterContext.createChildArrayContext(null, false);
+            return;
+        }
+        if (_itemFilter == TokenFilter.INCLUDE_ALL) {
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(forValue);
+            return;
+        }
+        _itemFilter = _filterContext.checkValue(_itemFilter);
+        if (_itemFilter == null) {
+            _filterContext = _filterContext.createChildArrayContext(null, false);
+            return;
+        }
+        if (_itemFilter != TokenFilter.INCLUDE_ALL) {
+            _itemFilter = _itemFilter.filterStartArray();
+        }
+        if (_itemFilter == TokenFilter.INCLUDE_ALL) {
+            _checkParentPath();
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(forValue);
+        } else {
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
+        }
+    }
+
+    @Override
+    public void writeStartArray(Object forValue, int size) throws IOException
+    {
+        if (_itemFilter == null) {
+            _filterContext = _filterContext.createChildArrayContext(null, false);
+            return;
+        }
+        if (_itemFilter == TokenFilter.INCLUDE_ALL) {
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(forValue, size);
+            return;
+        }
+        _itemFilter = _filterContext.checkValue(_itemFilter);
+        if (_itemFilter == null) {
+            _filterContext = _filterContext.createChildArrayContext(null, false);
+            return;
+        }
+        if (_itemFilter != TokenFilter.INCLUDE_ALL) {
+            _itemFilter = _itemFilter.filterStartArray();
+        }
+        if (_itemFilter == TokenFilter.INCLUDE_ALL) {
+            _checkParentPath();
+            _filterContext = _filterContext.createChildArrayContext(_itemFilter, true);
+            delegate.writeStartArray(forValue, size);
         } else {
             _filterContext = _filterContext.createChildArrayContext(_itemFilter, false);
         }
@@ -238,11 +323,79 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
             _checkParentPath();
             _filterContext = _filterContext.createChildObjectContext(f, true);
             delegate.writeStartObject();
+        } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildObjectContext(f, true);
+            delegate.writeStartObject();
         } else { // filter out
             _filterContext = _filterContext.createChildObjectContext(f, false);
         }
     }
-    
+
+    @Override
+    public void writeStartObject(Object forValue) throws IOException
+    {
+        if (_itemFilter == null) {
+            _filterContext = _filterContext.createChildObjectContext(_itemFilter, false);
+            return;
+        }
+        if (_itemFilter == TokenFilter.INCLUDE_ALL) {
+            _filterContext = _filterContext.createChildObjectContext(_itemFilter, true);
+            delegate.writeStartObject(forValue);
+            return;
+        }
+
+        TokenFilter f = _filterContext.checkValue(_itemFilter);
+        if (f == null) {
+            return;
+        }
+
+        if (f != TokenFilter.INCLUDE_ALL) {
+            f = f.filterStartObject();
+        }
+        if (f == TokenFilter.INCLUDE_ALL) {
+            _checkParentPath();
+            _filterContext = _filterContext.createChildObjectContext(f, true);
+            delegate.writeStartObject(forValue);
+        } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+            _checkParentPath(false /* isMatch */);
+            _filterContext = _filterContext.createChildObjectContext(f, true);
+            delegate.writeStartObject(forValue);
+        } else { // filter out
+            _filterContext = _filterContext.createChildObjectContext(f, false);
+        }
+    }
+
+    @Override
+    public void writeStartObject(Object forValue, int size) throws IOException
+    {
+        if (_itemFilter == null) {
+            _filterContext = _filterContext.createChildObjectContext(_itemFilter, false);
+            return;
+        }
+        if (_itemFilter == TokenFilter.INCLUDE_ALL) {
+            _filterContext = _filterContext.createChildObjectContext(_itemFilter, true);
+            delegate.writeStartObject(forValue, size);
+            return;
+        }
+
+        TokenFilter f = _filterContext.checkValue(_itemFilter);
+        if (f == null) {
+            return;
+        }
+
+        if (f != TokenFilter.INCLUDE_ALL) {
+            f = f.filterStartObject();
+        }
+        if (f == TokenFilter.INCLUDE_ALL) {
+            _checkParentPath();
+            _filterContext = _filterContext.createChildObjectContext(f, true);
+            delegate.writeStartObject(forValue, size);
+        } else {
+            _filterContext = _filterContext.createChildObjectContext(f, false);
+        }
+    }
+
     @Override
     public void writeEndObject() throws IOException
     {
@@ -292,6 +445,12 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
         }
     }
 
+    // 02-Dec-2019, tatu: Not sure what else to do... so use default impl from base class
+    @Override
+    public void writeFieldId(long id) throws IOException {
+        writeFieldName(Long.toString(id));
+    }
+
     /*
     /**********************************************************
     /* Public API, write methods, text/String values
@@ -315,7 +474,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeString(value);
     }
 
@@ -337,7 +496,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeString(text, offset, len);
     }
 
@@ -358,10 +517,32 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeString(value);
     }
 
+    @Override
+    public void writeString(Reader reader, int len) throws IOException {
+        if (_itemFilter == null) {
+            return;
+        }
+        if (_itemFilter != TokenFilter.INCLUDE_ALL) {
+            TokenFilter state = _filterContext.checkValue(_itemFilter);
+            if (state == null) {
+                return;
+            }
+            if (state != TokenFilter.INCLUDE_ALL) {
+                // [core#609]: do need to implement, but with 2.10.x TokenFilter no
+                // useful method to call so will be mostly unfiltered
+                if (!state.includeString(reader, len)) {
+                    return;
+                }
+            }
+            _checkParentPath();
+        }
+        delegate.writeString(reader, len);
+    }
+
     @Override
     public void writeRawUTF8String(byte[] text, int offset, int length) throws IOException
     {
@@ -397,7 +578,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     public void writeRaw(String text, int offset, int len) throws IOException
     {
         if (_checkRawValueWrite()) {
-            delegate.writeRaw(text);
+            delegate.writeRaw(text, offset, len);
         }
     }
 
@@ -429,7 +610,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     public void writeRawValue(String text) throws IOException
     {
         if (_checkRawValueWrite()) {
-            delegate.writeRaw(text);
+            delegate.writeRawValue(text);
         }
     }
 
@@ -437,7 +618,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     public void writeRawValue(String text, int offset, int len) throws IOException
     {
         if (_checkRawValueWrite()) {
-            delegate.writeRaw(text, offset, len);
+            delegate.writeRawValue(text, offset, len);
         }
     }
 
@@ -445,7 +626,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
     public void writeRawValue(char[] text, int offset, int len) throws IOException
     {
         if (_checkRawValueWrite()) {
-            delegate.writeRaw(text, offset, len);
+            delegate.writeRawValue(text, offset, len);
         }
     }
 
@@ -489,7 +670,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -510,7 +691,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -531,7 +712,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -552,7 +733,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -573,7 +754,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -594,7 +775,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -615,7 +796,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNumber(v);
     }
 
@@ -640,6 +821,27 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
         delegate.writeNumber(encodedValue);
     }
 
+    @Override
+    public void writeNumber(char[] encodedValueBuffer, int offset, int length) throws IOException, UnsupportedOperationException
+    {
+        if (_itemFilter == null) {
+            return;
+        }
+        if (_itemFilter != TokenFilter.INCLUDE_ALL) {
+            TokenFilter state = _filterContext.checkValue(_itemFilter);
+            if (state == null) {
+                return;
+            }
+            if (state != TokenFilter.INCLUDE_ALL) {
+                if (!state.includeRawValue()) { // close enough?
+                    return;
+                }
+            }
+            _checkParentPath();
+        }
+        delegate.writeNumber(encodedValueBuffer, offset, length);
+    }
+
     @Override
     public void writeBoolean(boolean v) throws IOException
     {
@@ -657,7 +859,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeBoolean(v);
     }
 
@@ -678,7 +880,7 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
                 }
             }
             _checkParentPath();
-        } 
+        }
         delegate.writeNull();
     }
 
@@ -801,13 +1003,23 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
 
     protected void _checkParentPath() throws IOException
     {
-        ++_matchCount;
+        _checkParentPath(true);
+    }
+
+    protected void  _checkParentPath(boolean isMatch) throws IOException
+    {
+        if (isMatch) {
+            ++_matchCount;
+        }
         // only need to construct path if parent wasn't written
-        if (_includePath) {
+        if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
             _filterContext.writePath(delegate);
+        } else if (_inclusion == Inclusion.INCLUDE_NON_NULL) {
+            // path has already been written, except for maybe field name
+            _filterContext.ensureFieldNameWritten(delegate);
         }
         // also: if no multiple matches desired, short-cut checks
-        if (!_allowMultipleMatches) {
+        if (isMatch && !_allowMultipleMatches) {
             // Mark parents as "skip" so that further check calls are not made
             _filterContext.skipParentChecks();
         }
@@ -817,16 +1029,17 @@ public class FilteringGeneratorDelegate extends JsonGeneratorDelegate
      * Specialized variant of {@link #_checkParentPath} used when checking
      * parent for a property name to be included with value: rules are slightly
      * different.
+     *
+     * @throws IOException If there is an issue with possible resulting read
      */
     protected void _checkPropertyParentPath() throws IOException
     {
         ++_matchCount;
-        if (_includePath) {
+        if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
             _filterContext.writePath(delegate);
-        } else if (_includeImmediateParent) {
-            // 21-Apr-2015, tatu: Note that there is no API to enable this currently...
-            //    retained for speculative future use
-            _filterContext.writeImmediatePath(delegate);
+        } else if (_inclusion == Inclusion.INCLUDE_NON_NULL) {
+            // path has already been written, except for maybe field name
+            _filterContext.ensureFieldNameWritten(delegate);
         }
 
         // also: if no multiple matches desired, short-cut checks
