@@ -6,6 +6,7 @@ import java.math.BigDecimal;
 import java.math.BigInteger;
 
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.filter.TokenFilter.Inclusion;
 import com.fasterxml.jackson.core.util.JsonParserDelegate;
 
 import static com.fasterxml.jackson.core.JsonTokenId.*;
@@ -46,19 +47,8 @@ public class FilteringParserDelegate extends JsonParserDelegate
      * done and only explicitly included entries are output; if `true` then
      * path from main level down to match is also included as necessary.
      */
-    protected boolean _includePath;
+    protected TokenFilter.Inclusion _inclusion;
 
-    /* NOTE: this feature is included in the first version (2.6), but
-     * there is no public API to enable it, yet, since there isn't an
-     * actual use case. But it seemed possible need could arise, which
-     * is feature has not yet been removed. If no use is found within
-     * first version or two, just remove.
-     * 
-     * Marked as deprecated since its status is uncertain.
-     */
-    @Deprecated
-    protected boolean _includeImmediateParent;
-    
     /*
     /**********************************************************
     /* State
@@ -111,15 +101,28 @@ public class FilteringParserDelegate extends JsonParserDelegate
     /**********************************************************
      */
 
+    @Deprecated
     public FilteringParserDelegate(JsonParser p, TokenFilter f,
             boolean includePath, boolean allowMultipleMatches)
+    {
+        this(p, f, includePath ? Inclusion.INCLUDE_ALL_AND_PATH : Inclusion.ONLY_INCLUDE_ALL, allowMultipleMatches);
+    }
+
+    /**
+     * @param p Parser to delegate calls to
+     * @param f Filter to use
+     * @param inclusion Definition of inclusion criteria
+     * @param allowMultipleMatches Whether to allow multiple matches
+     */
+    public FilteringParserDelegate(JsonParser p, TokenFilter f,
+            TokenFilter.Inclusion inclusion, boolean allowMultipleMatches)
     {
         super(p);
         rootFilter = f;
         // and this is the currently active filter for root values
         _itemFilter = f;
         _headContext = TokenFilterContext.createRootContext(f);
-        _includePath = includePath;
+        _inclusion = inclusion;
         _allowMultipleMatches = allowMultipleMatches;
     }
 
@@ -134,6 +137,8 @@ public class FilteringParserDelegate extends JsonParserDelegate
     /**
      * Accessor for finding number of matches, where specific token and sub-tree
      * starting (if structured type) are passed.
+     *
+     * @return Number of matches
      */
     public int getMatchCount() {
         return _matchCount;
@@ -148,9 +153,9 @@ public class FilteringParserDelegate extends JsonParserDelegate
     @Override public JsonToken getCurrentToken() { return _currToken; }
     @Override public JsonToken currentToken() { return _currToken; }
 
+    @Deprecated // since 2.12
     @Override public final int getCurrentTokenId() {
-        final JsonToken t = _currToken;
-        return (t == null) ? JsonTokenId.ID_NO_TOKEN : t.id();
+        return currentTokenId();
     }
     @Override public final int currentTokenId() {
         final JsonToken t = _currToken;
@@ -179,7 +184,7 @@ public class FilteringParserDelegate extends JsonParserDelegate
     public JsonStreamContext getParsingContext() {
         return _filterContext();
     }
-    
+
     // !!! TODO: Verify it works as expected: copied from standard JSON parser impl
     @Override
     public String getCurrentName() throws IOException {
@@ -191,6 +196,17 @@ public class FilteringParserDelegate extends JsonParserDelegate
         return ctxt.getCurrentName();
     }
 
+    // 2.13: IMPORTANT! Must override along with older getCurrentName()
+    @Override
+    public String currentName() throws IOException {
+        JsonStreamContext ctxt = _filterContext();
+        if (_currToken == JsonToken.START_OBJECT || _currToken == JsonToken.START_ARRAY) {
+            JsonStreamContext parent = ctxt.getParent();
+            return (parent == null) ? null : parent.getCurrentName();
+        }
+        return ctxt.getCurrentName();
+    }
+
     /*
     /**********************************************************
     /* Public API, token state overrides
@@ -210,10 +226,9 @@ public class FilteringParserDelegate extends JsonParserDelegate
 
     @Override
     public void overrideCurrentName(String name) {
-        /* 14-Apr-2015, tatu: Not sure whether this can be supported, and if so,
-         *    what to do with it... Delegation won't work for sure, so let's for
-         *    now throw an exception
-         */
+        // 14-Apr-2015, tatu: Not sure whether this can be supported, and if so,
+        //    what to do with it... Delegation won't work for sure, so let's for
+        //    now throw an exception
         throw new UnsupportedOperationException("Can not currently override name during filtering read");
     }
 
@@ -235,9 +250,10 @@ public class FilteringParserDelegate extends JsonParserDelegate
         // If all the conditions matches then check for scalar / non-scalar property
 
         if (!_allowMultipleMatches && (_currToken != null) && (_exposedContext == null)) {
-            // if scalar, and scalar not present in obj/array and !includePath and INCLUDE_ALL
-            // matched once, return null
-            if (_currToken.isScalarValue() && !_headContext.isStartHandled() && !_includePath
+            // if scalar, and scalar not present in obj/array and _inclusion == ONLY_INCLUDE_ALL
+            // and INCLUDE_ALL matched once, return null
+            if (_currToken.isScalarValue() && !_headContext.isStartHandled()
+                    && _inclusion == Inclusion.ONLY_INCLUDE_ALL
                     && (_itemFilter == TokenFilter.INCLUDE_ALL)) {
                 return (_currToken = null);
             }
@@ -263,15 +279,14 @@ public class FilteringParserDelegate extends JsonParserDelegate
                         return t;
                     }
 
+                    // 19-Jul-2021, tatu: [core#700]: following was commented out?!
                     // Almost! Most likely still have the current token;
-                    // with the sole exception of 
-                    /*
-                    t = delegate.getCurrentToken();
+                    // with the sole exception of FIELD_NAME
+                    t = delegate.currentToken();
                     if (t != JsonToken.FIELD_NAME) {
                         _currToken = t;
                         return t;
                     }
-                    */
                     break;
                 }
                 // If not, traverse down the context chain
@@ -318,11 +333,15 @@ public class FilteringParserDelegate extends JsonParserDelegate
             if (f == TokenFilter.INCLUDE_ALL) {
                 _headContext = _headContext.createChildArrayContext(f, true);
                 return (_currToken = t);
+            } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+                // TODO don't count as match?
+                _headContext = _headContext.createChildArrayContext(f, true);
+                return (_currToken = t);
             }
             _headContext = _headContext.createChildArrayContext(f, false);
-            
+
             // Also: only need buffering if parent path to be included
-            if (_includePath) {
+            if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
                 t = _nextTokenWithBuffering(_headContext);
                 if (t != null) {
                     _currToken = t;
@@ -354,10 +373,14 @@ public class FilteringParserDelegate extends JsonParserDelegate
             if (f == TokenFilter.INCLUDE_ALL) {
                 _headContext = _headContext.createChildObjectContext(f, true);
                 return (_currToken = t);
+            } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+                // TODO don't count as match?
+                _headContext = _headContext.createChildObjectContext(f, true);
+                return (_currToken = t);
             }
             _headContext = _headContext.createChildObjectContext(f, false);
             // Also: only need buffering if parent path to be included
-            if (_includePath) {
+            if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
                 t = _nextTokenWithBuffering(_headContext);
                 if (t != null) {
                     _currToken = t;
@@ -391,14 +414,6 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 f = _headContext.setFieldName(name);
                 if (f == TokenFilter.INCLUDE_ALL) {
                     _itemFilter = f;
-                    if (!_includePath) {
-                        // Minor twist here: if parent NOT included, may need to induce output of
-                        // surrounding START_OBJECT/END_OBJECT
-                        if (_includeImmediateParent && !_headContext.isStartHandled()) {
-                            t = _headContext.nextTokenToRead(); // returns START_OBJECT but also marks it handled
-                            _exposedContext = _headContext;
-                        }
-                    }
                     return (_currToken = t);
                 }
                 if (f == null) {
@@ -415,7 +430,7 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 _itemFilter = f;
                 if (f == TokenFilter.INCLUDE_ALL) {
                     if (_verifyAllowedMatches()) {
-                        if (_includePath) {
+                        if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
                             return (_currToken = t);
                         }
                     } else {
@@ -423,7 +438,7 @@ public class FilteringParserDelegate extends JsonParserDelegate
                         delegate.skipChildren();
                     }
                 }
-                if (_includePath) {
+                if (_inclusion != Inclusion.ONLY_INCLUDE_ALL) {
                     t = _nextTokenWithBuffering(_headContext);
                     if (t != null) {
                         _currToken = t;
@@ -455,12 +470,10 @@ public class FilteringParserDelegate extends JsonParserDelegate
         return _nextToken2();
     }
 
-    /**
-     * Offlined handling for cases where there was no buffered token to
-     * return, and the token read next could not be returned as-is,
-     * at least not yet, but where we have not yet established that
-     * buffering is needed.
-     */
+    // Offlined handling for cases where there was no buffered token to
+    // return, and the token read next could not be returned as-is,
+    // at least not yet, but where we have not yet established that
+    // buffering is needed.
     protected final JsonToken _nextToken2() throws IOException
     {
         main_loop:
@@ -496,10 +509,13 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 if (f == TokenFilter.INCLUDE_ALL) {
                     _headContext = _headContext.createChildArrayContext(f, true);
                     return (_currToken = t);
+                } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+                    _headContext = _headContext.createChildArrayContext(f, true);
+                    return (_currToken = t);
                 }
                 _headContext = _headContext.createChildArrayContext(f, false);
                 // but if we didn't figure it out yet, need to buffer possible events
-                if (_includePath) {
+                if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
                     t = _nextTokenWithBuffering(_headContext);
                     if (t != null) {
                         _currToken = t;
@@ -531,9 +547,12 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 if (f == TokenFilter.INCLUDE_ALL) {
                     _headContext = _headContext.createChildObjectContext(f, true);
                     return (_currToken = t);
+                } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+                    _headContext = _headContext.createChildObjectContext(f, true);
+                    return (_currToken = t);
                 }
                 _headContext = _headContext.createChildObjectContext(f, false);
-                if (_includePath) {
+                if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
                     t = _nextTokenWithBuffering(_headContext);
                     if (t != null) {
                         _currToken = t;
@@ -579,13 +598,17 @@ public class FilteringParserDelegate extends JsonParserDelegate
                     }
                     _itemFilter = f;
                     if (f == TokenFilter.INCLUDE_ALL) {
-                        if (_verifyAllowedMatches() && _includePath) {
-                            return (_currToken = t);
+                        if (_verifyAllowedMatches()) {
+                            if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
+                                return (_currToken = t);
+                            }
+                        } else {
+                            delegate.nextToken();
+                            delegate.skipChildren();
                         }
-//                        if (_includeImmediateParent) { ...
                         continue main_loop;
                     }
-                    if (_includePath) {
+                    if (_inclusion != Inclusion.ONLY_INCLUDE_ALL) {
                         t = _nextTokenWithBuffering(_headContext);
                         if (t != null) {
                             _currToken = t;
@@ -615,9 +638,7 @@ public class FilteringParserDelegate extends JsonParserDelegate
         }
     }
 
-    /**
-     * Method called when a new potentially included context is found.
-     */
+    // Method called when a new potentially included context is found.
     protected final JsonToken _nextTokenWithBuffering(final TokenFilterContext buffRoot)
         throws IOException
     {
@@ -647,6 +668,10 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 if (f == TokenFilter.INCLUDE_ALL) {
                     _headContext = _headContext.createChildArrayContext(f, true);
                     return _nextBuffered(buffRoot);
+                } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+                    // TODO don't count as match?
+                    _headContext = _headContext.createChildArrayContext(f, true);
+                    return _nextBuffered(buffRoot);
                 }
                 _headContext = _headContext.createChildArrayContext(f, false);
                 continue main_loop;
@@ -674,6 +699,10 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 if (f == TokenFilter.INCLUDE_ALL) {
                     _headContext = _headContext.createChildObjectContext(f, true);
                     return _nextBuffered(buffRoot);
+                } else if (f != null && _inclusion == Inclusion.INCLUDE_NON_NULL) {
+                    // TODO don't count as match?
+                    _headContext = _headContext.createChildArrayContext(f, true);
+                    return _nextBuffered(buffRoot);
                 }
                 _headContext = _headContext.createChildObjectContext(f, false);
                 continue main_loop;
@@ -836,11 +865,44 @@ public class FilteringParserDelegate extends JsonParserDelegate
     /**********************************************************
      */
 
-    @Override public String getText() throws IOException { return delegate.getText();  }
-    @Override public boolean hasTextCharacters() { return delegate.hasTextCharacters(); }
-    @Override public char[] getTextCharacters() throws IOException { return delegate.getTextCharacters(); }
-    @Override public int getTextLength() throws IOException { return delegate.getTextLength(); }
-    @Override public int getTextOffset() throws IOException { return delegate.getTextOffset(); }
+    // 19-Jul-2021, tatu: Cannot quite just delegate these methods due to oddity
+    //   of property name token, which may be buffered.
+
+    @Override public String getText() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName();
+        }
+        return delegate.getText();
+    }
+
+    @Override public boolean hasTextCharacters() {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return false;
+        }
+        return delegate.hasTextCharacters();
+    }
+
+    @Override public char[] getTextCharacters() throws IOException {
+        // Not optimal but is correct, unlike delegating (as underlying stream
+        // may point to something else due to buffering)
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName().toCharArray();
+        }
+        return delegate.getTextCharacters();
+    }
+
+    @Override public int getTextLength() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName().length();
+        }
+        return delegate.getTextLength();
+    }
+    @Override public int getTextOffset() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return 0;
+        }
+        return delegate.getTextOffset();
+    }
 
     /*
     /**********************************************************
@@ -895,9 +957,20 @@ public class FilteringParserDelegate extends JsonParserDelegate
     @Override public double getValueAsDouble(double defaultValue) throws IOException { return delegate.getValueAsDouble(defaultValue); }
     @Override public boolean getValueAsBoolean() throws IOException { return delegate.getValueAsBoolean(); }
     @Override public boolean getValueAsBoolean(boolean defaultValue) throws IOException { return delegate.getValueAsBoolean(defaultValue); }
-    @Override public String getValueAsString() throws IOException { return delegate.getValueAsString(); }
-    @Override public String getValueAsString(String defaultValue) throws IOException { return delegate.getValueAsString(defaultValue); }
-    
+
+    @Override public String getValueAsString() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName();
+        }
+        return delegate.getValueAsString();
+    }
+    @Override public String getValueAsString(String defaultValue) throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName();
+        }
+        return delegate.getValueAsString(defaultValue);
+    }
+
     /*
     /**********************************************************
     /* Public API, access to token values, other
@@ -921,5 +994,4 @@ public class FilteringParserDelegate extends JsonParserDelegate
         }
         return _headContext;
     }
-  
 }
