@@ -17,9 +17,9 @@ import com.fasterxml.jackson.core.io.NumberOutput;
 public class WriterBasedJsonGenerator
     extends JsonGeneratorImpl
 {
-    final protected static int SHORT_WRITE = 32;
+    protected final static int SHORT_WRITE = 32;
 
-    final protected static char[] HEX_CHARS = CharTypes.copyHexChars();
+    protected final static char[] HEX_CHARS = CharTypes.copyHexChars();
 
     /*
     /**********************************************************
@@ -27,7 +27,7 @@ public class WriterBasedJsonGenerator
     /**********************************************************
      */
 
-    final protected Writer _writer;
+    protected final Writer _writer;
 
     /**
      * Character used for quoting JSON Object property names
@@ -96,9 +96,7 @@ public class WriterBasedJsonGenerator
         this(ctxt, features, codec, w, JsonFactory.DEFAULT_QUOTE_CHAR);
     }
 
-    /**
-     * @since 2.10
-     */
+    // @since 2.10
     public WriterBasedJsonGenerator(IOContext ctxt, int features,
             ObjectCodec codec, Writer w,
             char quoteChar)
@@ -353,10 +351,8 @@ public class WriterBasedJsonGenerator
         _writeContext = _writeContext.clearAndGetParent();
     }
 
-    /**
-     * Specialized version of <code>_writeFieldName</code>, off-lined
-     * to keep the "fast path" as simple (and hopefully fast) as possible.
-     */
+    // Specialized version of <code>_writeFieldName</code>, off-lined
+    // to keep the "fast path" as simple (and hopefully fast) as possible.
     protected final void _writePPFieldName(String name, boolean commaBefore) throws IOException
     {
         if (commaBefore) {
@@ -430,37 +426,32 @@ public class WriterBasedJsonGenerator
     }
 
     @Override
-    public void writeString(Reader reader, int len) throws IOException {
+    public void writeString(Reader reader, final int len) throws IOException
+    {
         _verifyValueWrite(WRITE_STRING);
         if (reader == null) {
             _reportError("null reader");
             return; // just to block warnings by lgtm.com
         }
         int toRead = (len >= 0) ? len : Integer.MAX_VALUE;
-        //Add leading quote
-        if ((_outputTail + len) >= _outputEnd) {
+        // Add leading quote
+        if (_outputTail >= _outputEnd) {
             _flushBuffer();
         }
         _outputBuffer[_outputTail++] = _quoteChar;
 
         final char[] buf = _allocateCopyBuffer();
-        //read
         while (toRead > 0) {
             int toReadNow = Math.min(toRead, buf.length);
             int numRead = reader.read(buf, 0, toReadNow);
             if (numRead <= 0) {
                 break;
             }
-            if ((_outputTail + len) >= _outputEnd) {
-                _flushBuffer();
-            }
             _writeString(buf, 0, numRead);
-
-            //decrease tracker
             toRead -= numRead;
         }
-        //Add trailing quote
-        if ((_outputTail + len) >= _outputEnd) {
+        // Add trailing quote
+        if (_outputTail >= _outputEnd) {
             _flushBuffer();
         }
         _outputBuffer[_outputTail++] = _quoteChar;
@@ -983,9 +974,7 @@ public class WriterBasedJsonGenerator
     {
         super.close();
 
-        /* 05-Dec-2008, tatu: To add [JACKSON-27], need to close open
-         *   scopes.
-         */
+        // 05-Dec-2008, tatu: To add [JACKSON-27], need to close open scopes
         // First: let's see that we still have buffers...
         if (_outputBuffer != null
             && isEnabled(Feature.AUTO_CLOSE_JSON_CONTENT)) {
