@@ -536,6 +536,10 @@ public abstract class BaseTest
     }
 
     protected static String aposToQuotes(String json) {
+        return a2q(json);
+    }
+
+    protected static String a2q(String json) {
         return json.replace("'", "\"");
     }
 
@@ -605,6 +609,14 @@ public abstract class BaseTest
         return sb.toString();
     }
 
+    protected int[] calcQuads(String word) {
+        try {
+            return calcQuads(word.getBytes("UTF-8"));
+        } catch (IOException e) {
+            throw new RuntimeException(e);
+        }
+    }
+
     protected int[] calcQuads(byte[] wordBytes) {
         int blen = wordBytes.length;
         int[] result = new int[(blen + 3) / 4];
@@ -624,4 +636,27 @@ public abstract class BaseTest
         }
         return result;
     }
+
+    protected byte[] readResource(String ref)
+    {
+       ByteArrayOutputStream bytes = new ByteArrayOutputStream();
+       final byte[] buf = new byte[4000];
+
+       InputStream in = getClass().getResourceAsStream(ref);
+       if (in != null) {
+           try {
+               int len;
+               while ((len = in.read(buf)) > 0) {
+                   bytes.write(buf, 0, len);
+               }
+               in.close();
+           } catch (IOException e) {
+               throw new RuntimeException("Failed to read resource '"+ref+"': "+e);
+           }
+       }
+       if (bytes.size() == 0) {
+           throw new IllegalArgumentException("Failed to read resource '"+ref+"': empty resource?");
+       }
+       return bytes.toByteArray();
+    }
 }
