@@ -3,13 +3,16 @@ package com.fasterxml.jackson.core;
 import java.io.ByteArrayInputStream;
 import java.io.InputStream;
 
+import com.fasterxml.jackson.core.io.ContentReference;
+
 public class TestLocation extends BaseTest
 {
     static class Foobar { }
 
     public void testBasics()
     {
-        JsonLocation loc1 = new JsonLocation("src", 10L, 10L, 1, 2);
+        JsonLocation loc1 = new JsonLocation(_sourceRef("src"),
+                10L, 10L, 1, 2);
         JsonLocation loc2 = new JsonLocation(null, 10L, 10L, 3, 2);
         assertEquals(loc1, loc1);
         assertFalse(loc1.equals(null));
@@ -23,54 +26,66 @@ public class TestLocation extends BaseTest
 
     public void testBasicToString() throws Exception
     {
-        // no location:
+        // no location; presumed to be Binary due to defaulting
         assertEquals("[Source: UNKNOWN; line: 3, column: 2]",
                 new JsonLocation(null, 10L, 10L, 3, 2).toString());
 
         // Short String
         assertEquals("[Source: (String)\"string-source\"; line: 1, column: 2]",
-                new JsonLocation("string-source", 10L, 10L, 1, 2).toString());
+                new JsonLocation(_sourceRef("string-source"), 10L, 10L, 1, 2).toString());
 
         // Short char[]
         assertEquals("[Source: (char[])\"chars-source\"; line: 1, column: 2]",
-                new JsonLocation("chars-source".toCharArray(), 10L, 10L, 1, 2).toString());
+                new JsonLocation(_sourceRef("chars-source".toCharArray()), 10L, 10L, 1, 2).toString());
 
         // Short byte[]
         assertEquals("[Source: (byte[])\"bytes-source\"; line: 1, column: 2]",
-                new JsonLocation("bytes-source".getBytes("UTF-8"), 10L, 10L, 1, 2).toString());
+                new JsonLocation(_sourceRef("bytes-source".getBytes("UTF-8")),
+                        10L, 10L, 1, 2).toString());
 
         // InputStream
         assertEquals("[Source: (ByteArrayInputStream); line: 1, column: 2]",
-                new JsonLocation(new ByteArrayInputStream(new byte[0]), 10L, 10L, 1, 2).toString());
+                new JsonLocation(_sourceRef(new ByteArrayInputStream(new byte[0])),
+                        10L, 10L, 1, 2).toString());
 
         // Class<?> that specifies source type
         assertEquals("[Source: (InputStream); line: 1, column: 2]",
-                new JsonLocation(InputStream.class, 10L, 10L, 1, 2).toString());
+                new JsonLocation(_rawSourceRef(true, InputStream.class), 10L, 10L, 1, 2).toString());
 
         // misc other
         Foobar srcRef = new Foobar();
         assertEquals("[Source: ("+srcRef.getClass().getName()+"); line: 1, column: 2]",
-                new JsonLocation(srcRef, 10L, 10L, 1, 2).toString());
+                new JsonLocation(_rawSourceRef(true, srcRef), 10L, 10L, 1, 2).toString());
     }
 
     public void testTruncatedSource() throws Exception
     {
         StringBuilder sb = new StringBuilder();
-        for (int i = 0; i < JsonLocation.MAX_CONTENT_SNIPPET; ++i) {
+        for (int i = 0; i < ContentReference.DEFAULT_MAX_CONTENT_SNIPPET; ++i) {
             sb.append("x");
         }
         String main = sb.toString();
         String json = main + "yyy";
-        JsonLocation loc = new JsonLocation(json, 0L, 0L, 1, 1);
+        JsonLocation loc = new JsonLocation(_sourceRef(json), 0L, 0L, 1, 1);
         String desc = loc.sourceDescription();
         assertEquals(String.format("(String)\"%s\"[truncated 3 chars]", main), desc);
 
         // and same with bytes
-        loc = new JsonLocation(json.getBytes("UTF-8"), 0L, 0L, 1, 1);
+        loc = new JsonLocation(_sourceRef(json.getBytes("UTF-8")), 0L, 0L, 1, 1);
         desc = loc.sourceDescription();
         assertEquals(String.format("(byte[])\"%s\"[truncated 3 bytes]", main), desc);
     }
 
+    // for [jackson-core#658]
+    public void testEscapeNonPrintable() throws Exception
+    {
+        final String DOC = "[ \"tab:[\t]/null:[\0]\" ]";
+        JsonLocation loc = new JsonLocation(_sourceRef(DOC), 0L, 0L, -1, -1);
+        final String sourceDesc = loc.sourceDescription();
+        assertEquals(String.format("(String)\"[ \"tab:[%s]/null:[%s]\" ]\"",
+                "\\u0009", "\\u0000"), sourceDesc);
+    }
+
     // for [jackson-core#356]
     public void testDisableSourceInclusion() throws Exception
     {
@@ -86,7 +101,7 @@ public class TestLocation extends BaseTest
         } catch (JsonParseException e) {
             verifyException(e, "unrecognized token");
             JsonLocation loc = e.getLocation();
-            assertNull(loc.getSourceRef());
+            assertNull(loc.contentReference().getRawContent());
             assertEquals("UNKNOWN", loc.sourceDescription());
         }
         p.close();
@@ -100,9 +115,29 @@ public class TestLocation extends BaseTest
         } catch (JsonParseException e) {
             verifyException(e, "unrecognized token");
             JsonLocation loc = e.getLocation();
-            assertNull(loc.getSourceRef());
+            assertNull(loc.contentReference().getRawContent());
             assertEquals("UNKNOWN", loc.sourceDescription());
         }
         p.close();
     }
+
+    private ContentReference _sourceRef(String rawSrc) {
+        return ContentReference.construct(true, rawSrc, 0, rawSrc.length());
+    }
+
+    private ContentReference _sourceRef(char[] rawSrc) {
+        return ContentReference.construct(true, rawSrc, 0, rawSrc.length);
+    }
+
+    private ContentReference _sourceRef(byte[] rawSrc) {
+        return ContentReference.construct(true, rawSrc, 0, rawSrc.length);
+    }
+
+    private ContentReference _sourceRef(InputStream rawSrc) {
+        return ContentReference.construct(true, rawSrc, -1, -1);
+    }
+
+    private ContentReference _rawSourceRef(boolean textual, Object rawSrc) {
+        return ContentReference.rawReference(textual, rawSrc);
+    }
 }
