@@ -7,7 +7,8 @@ public class TestIOContext
 {
     public void testAllocations() throws Exception
     {
-        IOContext ctxt = new IOContext(new BufferRecycler(), "N/A", true);
+        IOContext ctxt = new IOContext(new BufferRecycler(),
+                ContentReference.rawReference("N/A"), true);
 
         /* I/O Read buffer */
 
