@@ -6,7 +6,6 @@ import java.util.Random;
 import static org.junit.Assert.*;
 
 import com.fasterxml.jackson.core.*;
-import com.fasterxml.jackson.core.util.BufferRecyclers;
 
 public class TestJsonStringEncoder
     extends com.fasterxml.jackson.core.BaseTest
@@ -18,6 +17,12 @@ public class TestJsonStringEncoder
         assertArrayEquals("foobar".toCharArray(), result);
         result = encoder.quoteAsString("\"x\"");
         assertArrayEquals("\\\"x\\\"".toCharArray(), result);
+
+        // and simply for sake of code coverage
+        result = encoder.quoteAsString(new StringBuilder("foobar"));
+        assertArrayEquals("foobar".toCharArray(), result);
+        result = encoder.quoteAsString(new StringBuilder("\"x\""));
+        assertArrayEquals("\\\"x\\\"".toCharArray(), result);
     }
 
     public void testQuoteCharSequenceAsString() throws Exception
@@ -25,12 +30,12 @@ public class TestJsonStringEncoder
         StringBuilder output = new StringBuilder();
         StringBuilder builder = new StringBuilder();
         builder.append("foobar");
-        BufferRecyclers.quoteAsJsonText(builder, output);
+        JsonStringEncoder.getInstance().quoteAsString(builder, output);
         assertEquals("foobar", output.toString());
         builder.setLength(0);
         output.setLength(0);
         builder.append("\"x\"");
-        BufferRecyclers.quoteAsJsonText(builder, output);
+        JsonStringEncoder.getInstance().quoteAsString(builder, output);
         assertEquals("\\\"x\\\"", output.toString());
     }
 
@@ -49,7 +54,6 @@ public class TestJsonStringEncoder
         char[] result = encoder.quoteAsString(input);
         assertEquals(2*input.length(), result.length);
         assertEquals(exp, new String(result));
-        
     }
 
     public void testQuoteLongCharSequenceAsString() throws Exception
@@ -62,7 +66,7 @@ public class TestJsonStringEncoder
             sb2.append("\\\"");
         }
         String exp = sb2.toString();
-        BufferRecyclers.quoteAsJsonText(input, output);
+        JsonStringEncoder.getInstance().quoteAsString(input, output);
         assertEquals(2*input.length(), output.length());
         assertEquals(exp, output.toString());
 
@@ -101,14 +105,17 @@ public class TestJsonStringEncoder
                 generateRandom(39000)
         };
         for (String str : strings) {
-            assertArrayEquals(str.getBytes("UTF-8"), encoder.encodeAsUTF8(str));
+            final byte[] exp = str.getBytes("UTF-8");
+            assertArrayEquals(exp, encoder.encodeAsUTF8(str));
+            // and for 2.x code coverage (only
+            assertArrayEquals(exp, encoder.encodeAsUTF8((CharSequence)str));
         }
     }
 
     public void testCtrlChars() throws Exception
     {
         char[] input = new char[] { 0, 1, 2, 3, 4 };
-        char[] quoted = BufferRecyclers.quoteAsJsonText(new String(input));
+        char[] quoted = JsonStringEncoder.getInstance().quoteAsString(new String(input));
         assertEquals("\\u0000\\u0001\\u0002\\u0003\\u0004", new String(quoted));
     }
 
@@ -119,17 +126,59 @@ public class TestJsonStringEncoder
         StringBuilder builder = new StringBuilder();
         builder.append(input);
         StringBuilder output = new StringBuilder();
-        BufferRecyclers.quoteAsJsonText(builder, output);
+        JsonStringEncoder.getInstance().quoteAsString(builder, output);
         assertEquals("\\u0000\\u0001\\u0002\\u0003\\u0004", output.toString());
     }
 
+    // [core#712]: simple sanity checks for calculation logic
+    public void testByteBufferDefaultSize()
+    {
+        // byte size is simple, x2 except below buffer size 24
+        assertEquals(JsonStringEncoder.MIN_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(1));
+        assertEquals(JsonStringEncoder.MIN_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(11));
+
+        assertEquals(36, JsonStringEncoder._initialByteBufSize(20));
+        assertEquals(73, JsonStringEncoder._initialByteBufSize(45));
+        assertEquals(1506, JsonStringEncoder._initialByteBufSize(1000));
+        assertEquals(9006, JsonStringEncoder._initialByteBufSize(6000));
+
+        // and up to max initial size
+        assertEquals(JsonStringEncoder.MAX_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(JsonStringEncoder.MAX_BYTE_BUFFER_SIZE + 1));
+        assertEquals(JsonStringEncoder.MAX_BYTE_BUFFER_SIZE,
+                JsonStringEncoder._initialByteBufSize(999999));
+    }
+
+    // [core#712]: simple sanity checks for calculation logic
+    public void testCharBufferDefaultSize()
+    {
+        // char[] bit more complex, starts with minimum size of 16
+        assertEquals(JsonStringEncoder.MIN_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(1));
+        assertEquals(JsonStringEncoder.MIN_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(8));
+
+        // and then grows by ~5%
+        assertEquals(62, JsonStringEncoder._initialCharBufSize(50));
+        assertEquals(118, JsonStringEncoder._initialCharBufSize(100));
+        assertEquals(1131, JsonStringEncoder._initialCharBufSize(1000));
+        assertEquals(9000, JsonStringEncoder._initialCharBufSize(8000));
+
+        // up to max, simi
+        assertEquals(JsonStringEncoder.MAX_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(32000));
+        assertEquals(JsonStringEncoder.MAX_CHAR_BUFFER_SIZE,
+                JsonStringEncoder._initialCharBufSize(900000));
+    }
+
     /*
-    /**********************************************************
+    /**********************************************************************
     /* Helper methods
-    /**********************************************************
+    /**********************************************************************
      */
 
-    
     private String generateRandom(int length)
     {
         StringBuilder sb = new StringBuilder(length);
