@@ -181,7 +181,7 @@ public class NonStandardParserFeaturesTest
             /*BigDecimal dec =*/ p.getDecimalValue();
             fail("Should fail when trying to access NaN as BigDecimal");
         } catch (NumberFormatException e) {
-            verifyException(e, "can not be represented as BigDecimal");
+            verifyException(e, "can not be represented as `java.math.BigDecimal`");
         }
        
         assertToken(JsonToken.END_ARRAY, p.nextToken());
